//layout toc secbold title(subsection) link(subsection)
//layout bottombar toc

//titlepage --------------------------------------------------------------------
//title Agent based models in Mata: Modelling aggregate processes, like the spread of a disease

/*txt

{center:Maarten Buis}
{center:University of Konstanz}

{center:maarten.buis@uni.kn}
txt*/
//endtitlepage

/*toctitle
Table of Contents
toctitle*/

//section What is an Agent Based Model?
//slide ------------------------------------------------------------------------
//title What is an Agent Based Model?

/*txt
{pstd}
An Agent Based Model is a simulation in which agents, that each follow simple rules, 
interact with one another and thus produce a often surprising outcome at the macro 
level. 

{pstd}
The purpose of an ABM is to explore mechanisms through which actions of the 
individual agents add up to a macro outcome, by varying the rules that agents have
to follow or varying with whom the agent can interact. 
txt*/

//endslide ---------------------------------------------------------------------



//slide ------------------------------------------------------------------------
//title Example: The spread of a disease 

/*txt
{pstd}
In this model each agent can be in one of three states:

{pmore}{it:Susceptible}: the agent is healthy, but can get the disease.{p_end}
{pmore}{it:Infectious}: the agent has the disease and can spread it to others.{p_end}
{pmore}{it:Recovered}: the agent has had the disease and is now immune.{p_end}

{pstd}
In the first round one or more agents get infected.

{pstd}
In all subsequent rounds:

{pmore}- All infectious agents have a chance to become recovered, and{p_end}
{pmore}- all recovered agents get a chance to become susceptible.{p_end}
{pmore}- All agents that are still infectious contact a random number of other 
agents{p_end}
{pmore}- Each of these contact has a chance of resulting in an infection if 
someone is susceptible.
txt*/

//ex
set seed 1234567
set scheme s1color
clear mata
drop _all
run sir_main.do

mata:

model = sir()
model.N(2000)
model.tdim(150)
model.outbreak(5)
model.mcontacts(4)
model.transmissibility(0.045)
model.mindur(10)
model.meandur(14)
model.pr_loss(0)
model.run()
model.export_sir()
end

twoway line S I R t,  name(sir1, replace)          ///
   ylabel(,angle(0)) legend(rows(1)) lwidth(*1.5 ..) 
   
drop _all
//endex
//graph sir1

/*txt
{pstd}
The number 0.045 for transmissibility was chosen to get a /*digr*/ of 2.5, which
in line with COVID-19.
txt*/

//file sir_chks.do       
//file sir_cscript.do    
//file sir_export.do     
//file sir_locals.do     
//file sir_main.do       
//file sir_set_pars.do   
//file sir_sim.do   
//file abm_pop.mata      
//file abm_pop.sthlp     
//file abm_pop_add.sthlp 
//file abm_pop_extract.sthlp
//file abm_pop_get.sthlp 
//file abm_pop_k.sthlp   
//file abm_pop_N.sthlp   
//file abm_pop_put.sthlp 
//file abm_pop_setup.sthlp
   
//endslide --------------------------------------------------------------------- 

//digr -------------------------------------------------------------------------
//label R0
//title R0

/*txt
{pstd}
14 days is a number that has floated around a lot for how long someone is 
infectious when having COVID-19. 

{pstd}
On average 4 contacts is just a convenient number 

{pstd}
The transmissibility is then chosen to make the R0 close to 2.5, another number
that has floated around a lot for COVID-19.

{pmore}
If someone is 14 days infectious, and contacts 4 persons each day, then there 
are 14*4=56 opportunities to infect someone. 

{pmore}
These should result in 2.5 infections, so the chance of a contact leading to an
infection is 2.5/56 = 0.045
txt*/

//enddigr ----------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Social distancing and masks

/*txt
{pstd}
The purpose of social distancing and encouraging masks is to reduce the chance
of a contact leading to an infection.

{pstd}
We can change the model such that these policies reduce the transmissibility by
a quarter
txt*/

//ex
mata:
model.transmissibility(0.03)
model.run()
model.export_sir()
end

twoway line S I R t,  name(sir2, replace)          ///
   ylabel(,angle(0)) legend(rows(1)) lwidth(*1.5 ..) 
drop _all
//endex
//graph sir2

/*txt
{pstd}
We can similarly study the effect of a lock-down, which reduces the number of 
people we contact

{pstd}
or we can adept the model such that we look at how contact tracing might 
influence the spread of the disease
txt*/
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Losing immunity

/*txt
{pstd}
Recently, there have an increasing number of news stories about that immunity
against COVID-19 does not persist 

{pstd}
We can change to model to see how that impacts the spread of the disease
txt*/

//ex
mata:
model.transmissibility(0.045)
model.pr_loss(0.02)
model.tdim(250)
model.run()
model.export_sir()
end

twoway line S I R t,  name(sir3, replace)          ///
   ylabel(,angle(0)) legend(rows(1)) lwidth(*1.5 ..) 
drop _all
//endex
//graph sir3

//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Adding a network

/*txt
{pstd}
The model thus far does not need a simulation; one can compute them with a set
of differential equations. 

{pstd}
In Stata you can do that with {cmd:epimodels} by Sergiy Radyakin and Paolo Verme

{pstd}
However, the advantage of Agent Based Models is that it is much easier to expand
the model.

{pstd}
For example, what if we don't believe that people contact each other randomly 
but instead it is a network that determines who contacts who. 

{pstd}
A common model that works fairly well for a lot of social networks is a small 
world network:

{pmore}
We assume all agents are on a circle, and that each agent is connected to the 4
closes agents.

{pmore}
However, a small number of these connections are replaced by a connection to a
random agent
txt*/

//ex
clear mata
run nw_main.do
mata:
model = sir_nw()
model.N(100)
model.tdim(10)
model.outbreak(2)
model.degree(4)
model.pr_long(.05)
model.transmissibility(0.1)
model.mindur(5)
model.meandur(6)
model.pr_loss(0.00)
model.run()
model.export_nw()
end

gen x_ego = cos((ego-1)/`n'*2*_pi)*(1+mod(ego,2)*.2)
gen y_ego = sin((ego-1)/`n'*2*_pi)*(1+mod(ego,2)*.2)
gen x_alter = cos((alter-1)/`n'*2*_pi)*(1+mod(alter,2)*.2)
gen y_alter = sin((alter-1)/`n'*2*_pi)*(1+mod(alter,2)*.2)
    

bys ego (alter) : gen first = _n == 1

forvalues t = 1/10{
    twoway pcspike y_ego x_ego y_alter x_alter,                ///
                  lcolor(gs8)                               || ///
        scatter y_ego x_ego if first & state`t'==1 ,           ///
                   mcolor(yellow) msymbol(O)                || ///
        scatter y_ego x_ego if first & state`t'==2 ,           ///
                   mcolor(red) msymbol(O)                   || ///
        scatter y_ego x_ego if first & state`t'==3 ,           ///
                   mcolor(black) msymbol(Oh)                   ///         
                   aspect(1) xscale(off) yscale(off)           ///
                   legend(order(2 "infected"                   ///
                                3 "susceptible"                ///
                                4 "recovered") rows(1))        ///
                   name(t`t', replace) title(Time `t')     
}
drop _all
//endex
//graph t1
//graph t2
//graph t3
//graph t4
//graph t5
//graph t6
//graph t7
//graph t8
//graph t9
//graph t10

//file nw_chks.do        
//file nw_cscript.do     
//file nw_export.do      
//file nw_locals.do      
//file nw_main.do        
//file nw_set_pars.do    
//file nw_sim.do  
//file abm_nw.mata       
//file abm_nw.sthlp      
//file abm_nw_add_edge.sthlp
//file abm_nw_add_node.sthlp
//file abm_nw_change_weight.sthlp
//file abm_nw_clear.sthlp
//file abm_nw_copy_nw.sthlp
//file abm_nw_directed.sthlp
//file abm_nw_error.sthlp
//file abm_nw_export_adjmat.sthlp
//file abm_nw_export_edgelist.sthlp
//file abm_nw_from_adjlist.sthlp
//file abm_nw_from_adjmatrix.sthlp
//file abm_nw_from_edgelist.sthlp
//file abm_nw_neighbours.sthlp
//file abm_nw_no_edge.sthlp
//file abm_nw_N_edges.sthlp
//file abm_nw_N_nodes.sthlp
//file abm_nw_protected.sthlp
//file abm_nw_random.sthlp
//file abm_nw_randomit.sthlp
//file abm_nw_remove_edge.sthlp
//file abm_nw_remove_node.sthlp
//file abm_nw_return_node.sthlp
//file abm_nw_rewire.sthlp
//file abm_nw_schedule.sthlp
//file abm_nw_setup.sthlp
//file abm_nw_store_nw.sthlp
//file abm_nw_sw.sthlp   
//file abm_nw_tdim.sthlp 
//file abm_nw_weight.sthlp
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Avoiding unnecesary travel

/*txt
{pstd}
Close knit networks tend to slow down the spread of a disease, while the long 
distance ties tend 

{pstd}
So that is another way in which advise against unnecesary travel or "social buble"
rules in countries like Belgium works

txt*/

//ex
mata:
model.N(2000)
model.tdim(150)
model.outbreak(10)
model.degree(10)
model.pr_long(.2)
model.transmissibility(.018)
model.mindur(10)
model.meandur(14)
model.pr_loss(0.00)
model.run()
model.export_sir()
end

twoway line S I R t,  name(sir4, replace)          ///
   ylabel(,angle(0)) legend(rows(1)) lwidth(*1.5 ..) 
drop _all

mata:
model.pr_long(.05)
model.run()
model.export_sir()
end

twoway line S I R t,  name(sir5, replace)          ///
   ylabel(,angle(0)) legend(rows(1)) lwidth(*1.5 ..) 

drop _all
//endex
//graph sir4
//graph sir5
//endslide ---------------------------------------------------------------------

//section How to implement an ABM in Mata
//slide ------------------------------------------------------------------------
//title Where to start

/*txt
{pstd}
We have a population of agents to keep track of

{pstd}
We need to repeatedly apply rules to them

{pstd}
Applying rules can be done by a function, but that function needs access to 
settings and the population. 

{pstd}
This is a good case for a class.

{pstd}
A class can be thought of as a tray with named slots for functions and data

{pstd}
The functions are local to the class

{pstd}
Each function has access to all material stored in the class

txt*/

//ex
clear mata

mata:

class arithmatic {

    real scalar a
    real scalar b
    
    real scalar sum()
    real scalar prod()
    void input()
    void run()
}

void arithmatic::input(real scalar aval, real scalar bval)
{
    a = aval
    b = bval
}

real scalar arithmatic::sum()
{
    return(a + b)
}

real scalar arithmatic::prod()
{
    return(a * b)
}

void arithmatic::run()
{
    sum()
    prod()
}

// how the user interacts with this program
math = arithmatic()
math.input(2,4)
math.run()
end
//endex
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Basic SIR model

/*txt
{pstd}
Lets take a look at the code for the basic SIR model: 

//codefile sir_main.do here

{pstd}
The population is stored is stored in another class {help abm_pop}, which is 
available from {browse "https://github.com/maartenteaches/abm_pop"}.

{pstd}
Whatever is stored in {cmd:abm_pop} is assumed to persist until you store 
something else. So you only have to store the disease state of an agent when it
changes. This can save a lot of reading and writing and make the simulation run
quicker.

{pstd}
There are various checks perfomed and we can see those 

//codefile sir_chks.do here

{pstd}
The functions that set the parameters are shown 

//codefile sir_set_pars.do here 

{pstd}
The main simulation functions are

//codefile sir_sim.do here

{pstd}
The "variables" in {it:agents} are actually numbered and not named. To make it 
easier to read the code I use local macros defined in

//codefile sir_locals.do here

{pstd}
The functions that allow you to export the results to Stata are

//codefile sir_export.do here
txt*/
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title SIR model in a network

/*txt
{pstd}
To add the network we make use the {cmd:abm_nw} class, which is available from
{browse "https://github.com/maartenteaches/abm_nw"}.

{pstd}
This takes care of creating the network and keeps track of who can contact who.

{pstd}
With that the changes required are fairly small

{pstd}
The class definition is 

//codefile nw_main.do here

{pstd}
Checks and setting parameters are pretty much the same

{pstd}
The main simulation functions are

//codefile nw_sim.do here
txt*/
//endslide ---------------------------------------------------------------------

//section Final remarks
//slide ------------------------------------------------------------------------
//title Conclusion

/*txt
{pstd}
Implementing an ABM is doable in Mata 

{pstd}
We have seen two "helper classes": 

{pmore}{cmd:abm_pop} for storing a population of agents{p_end}
{pmore}{cmd:abm_nw} for managing a network{p_end}

{pstd}
There is also the {cmd:abm_grid} class to manage a grid, like a chessboard, which
is available from {browse "https://github.com/maartenteaches/abm_grid"}.
txt*/
//endslide ---------------------------------------------------------------------


//slide ------------------------------------------------------------------------
//title Alternative platforms

/*txt
{pstd}
There are various dedicated environments for creating ABMs. 

{pmore}
{bf:netlogo} (netlogo) {browse "http://ccl.northwestern.edu/netlogo/"}

{pmore}
{bf:repast} (java, python, C++, and others) {browse "https://repast.github.io/"}

{pmore}
{bf:mason} (java) {browse "https://cs.gmu.edu/~eclab/projects/mason/"}

{pmore}
{bf:mesa} (python) {browse "https://mesa.readthedocs.io/"}

{pstd}
People who primarily create ABMs are not going to move away from those, and 
rightly so.

{pstd}
The toolkits discussed in this presentation are aimed at people who 
primarily use Stata and occationally want to make an ABM.
txt*/

//endslide ---------------------------------------------------------------------
